#include "render.h"
#include "engine_global.h"
#include "scene.h"
#include "utils/debug.h"
#include "utils/list.h"

static List g_render_objects = {}; //!< List of all registered rendering objects

void InitializeRenderingSubsystem() {
    g_render_objects = list_from_type(Renderable); // Allocate list of renderable typeclass wrappers
}

void CleanupRenderingSubsystem() {
    list_empty(&g_render_objects); // deallocate renderable list, rendering subsystem is not responsible for freeing underlying memory
}

void AddRenderable(Renderable renderable) {
    ASSERT_RETURN(list_find(&g_render_objects, &renderable) == g_render_objects.len,, "AddRenderable: Argument is already registered, cannot register twice");
    list_add(&g_render_objects, &renderable);
}

void RemoveRenderable(Renderable renderable) {
    size_t idx = list_find(&g_render_objects, &renderable);
    ASSERT_RETURN(idx != g_render_objects.len,, "RemoveRenderable: Argument is not registered, cannot unregister.");
    list_erase(&g_render_objects, idx);
}

void RenderNextFrame() {
    BeginDrawing();
    ClearBackground(DARKGRAY);
    // get current camera
    CameraNode *camera = GetMainScene()->main_camera;
    // can't draw anything if there is no camera
    if(camera != NULL) {
        // render 3D scene
        BeginMode3D(CameraNodeGetCamera(camera));
        list_foreach(Renderable *,object, &g_render_objects)
            object->tc->draw(object->data);
        DrawGrid(100, 1.f);
        EndMode3D();
    }
    DrawFPS(20, 20);
    EndDrawing();
}
